var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};

// src/core/unplugin.ts
import events from "events";
import { createUnplugin } from "unplugin";

// src/core/proxy.ts
import httpProxy from "http-proxy";

// src/core/json2ts.ts
import * as fs from "fs";
import JsonToTS from "json-to-ts";
import MagicString from "magic-string";
import { pascalCase } from "change-case";
import { toArray } from "@imyangyong/utils";
import parse from "url-parse";
function parseUrl(req) {
  const parsedUrl = parse(req.url).pathname.split("/").map((name) => `${(name[0] || "").toUpperCase()}${name.slice(1)}`).join("");
  return `${req.method.toString().toUpperCase()}_${pascalCase(parsedUrl)}.d.ts`;
}
function isIgnore(req, ignore) {
  if (ignore) {
    const ignoreList = toArray(ignore);
    for (const ignore2 of ignoreList) {
      if (ignore2.test(req.url))
        return true;
    }
  }
  return false;
}
function writeFileSyncRecursive(filename, content, charset = "utf-8") {
  let filepath = filename.replace(/\\/g, "/");
  let root = "";
  if (filepath[0] === "/") {
    root = "/";
    filepath = filepath.slice(1);
  } else if (filepath[1] === ":") {
    root = filepath.slice(0, 3);
    filepath = filepath.slice(3);
  }
  const folders = filepath.split("/").slice(0, -1);
  folders.reduce(
    (acc, folder) => {
      const folderPath = `${acc + folder}/`;
      if (!fs.existsSync(folderPath))
        fs.mkdirSync(folderPath);
      return folderPath;
    },
    root
    // first 'acc', important
  );
  fs.writeFileSync(root + filepath, content, charset);
}
var transformJSONToTs = (req, chunks, options) => {
  const {
    suffix = "ResponseType",
    encoding = "utf-8",
    rootDir = "@types",
    overwrite = true,
    declare = true,
    ignore = null
  } = options;
  if (isIgnore(req, ignore))
    return;
  const name = parseUrl(req);
  let dir = rootDir;
  if (rootDir.endsWith("/")) {
    const splitDir = rootDir.split("/");
    dir = splitDir.slice(0, splitDir.length - 1).join("");
  }
  dir = `${dir}/${name}`;
  const isFileExist = fs.existsSync(dir);
  if (overwrite || !isFileExist) {
    try {
      const json = JSON.parse(Buffer.concat(chunks).toString(encoding));
      const code = new MagicString(JsonToTS(json).toLocaleString().replaceAll("},interface", "}\ninterface"));
      const typename = name.split("_");
      code.replace("interface RootObject", `${declare ? "declare" : "export"} interface ${typename[0]}${typename[1].replace(".d.ts", "")}${suffix}`);
      writeFileSyncRecursive(dir, code.toString());
    } catch (err) {
      console.log(`[proxy-json2ts Error] ${err}`);
    }
  }
};

// src/core/proxy.ts
var proxy = httpProxy.createProxyServer();
var chunksMap = /* @__PURE__ */ new Map();
var optionsMap = /* @__PURE__ */ new Map();
proxy.on("proxyRes", (proxyRes, req) => {
  chunksMap.set(req.url, []);
  proxyRes.on("data", (chunk) => {
    chunksMap.get(req.url).push(chunk);
  });
  proxyRes.on("end", () => {
    var _a;
    if ((_a = optionsMap.get(req.url)) == null ? void 0 : _a.json2ts)
      transformJSONToTs(req, chunksMap.get(req.url), optionsMap.get(req.url).json2ts);
  });
});
var proxyWeb = (req, res, option) => {
  optionsMap.set(req.url, option);
  proxy.web(req, res, __spreadValues({
    changeOrigin: true
  }, option));
};

// src/core/unplugin.ts
var ee = events.EventEmitter;
var unplugin_default = createUnplugin((options) => {
  return {
    name: "unplugin-dev-proxy",
    vite: {
      configureServer(server) {
        ee.setMaxListeners(0);
        if (options) {
          for (const [route, option] of Object.entries(options.proxy)) {
            server.middlewares.use(route, (req, res) => {
              proxyWeb(req, res, option);
            });
          }
        }
      }
    }
  };
});

export {
  unplugin_default
};
